"""
Buscar um genero
"""
# pylint: disable=duplicate-code
# Python
from logging import Logger
from dataclasses import asdict

# Apps
from kernel_catalogo_videos.genres.domain.entities import Genre
from kernel_catalogo_videos.core.application.use_case import UseCase
from kernel_catalogo_videos.genres.domain.repositories import GenreRepository
from kernel_catalogo_videos.genres.application.use_cases.dto import GenreOutputMapper
from kernel_catalogo_videos.genres.application.use_cases.get.input import GetGenreInput
from kernel_catalogo_videos.genres.application.use_cases.get.output import GetGenreOutput


class GetGenreUseCase(UseCase[GetGenreInput, GetGenreOutput]):
    """
    Classe para criar um genero
    """

    repo: GenreRepository

    def __init__(self, repo: GenreRepository, logger: Logger | None = None) -> None:
        self.repo = repo
        self.logger = logger

    def execute(self, input_params: GetGenreInput) -> GetGenreOutput:
        if self.logger:
            self.logger.info("get.genre.usecase", message="Initial Payload", input_params=asdict(input_params))

        genre = self.repo.find_by_id(input_params.id)

        if self.logger:
            self.logger.info("get.genre.usecase", message="Entity founded")

        return self.__to_output(genre=genre)

    def __to_output(self, genre: Genre):
        return GenreOutputMapper.to_output(klass=GetGenreOutput, genre=genre)



# Python
import abc
from typing import Generic, TypeVar

Input = TypeVar("Input")
Output = TypeVar("Output")


class UseCase(Generic[Input, Output], abc.ABC):
    @abc.abstractmethod
    def execute(self, input_params: Input) -> Output:
        raise NotImplementedError()



"""
Caso de uso para atualizar um video
"""
# pylint: disable=duplicate-code

# Python
from logging import Logger
from dataclasses import asdict

# Apps
from kernel_catalogo_videos.core.utils import ACTIVE_STATUS, INACTIVE_STATUS
from kernel_catalogo_videos.videos.domain.entities import Video
from kernel_catalogo_videos.core.application.use_case import UseCase
from kernel_catalogo_videos.videos.domain.repositories import VideoRepository
from kernel_catalogo_videos.videos.application.use_cases.dto import VideoOutputMapper
from kernel_catalogo_videos.videos.application.use_cases.update.input import UpdateVideoInput
from kernel_catalogo_videos.videos.application.use_cases.update.output import UpdateVideoOutput


class UpdateVideoUseCase(UseCase[UpdateVideoInput, UpdateVideoOutput]):
    """
    Atualizar um video
    """

    repo: VideoRepository

    def __init__(self, repo: VideoRepository, logger: Logger | None = None) -> None:
        self.repo = repo
        self.logger = logger

    def execute(self, input_params: UpdateVideoInput) -> UpdateVideoOutput:
        if self.logger:
            self.logger.info("update.video.usecase", message="Initial Payload", input_params=asdict(input_params))

        entity = self.repo.find_by_id(input_params.id)

        if self.logger:
            self.logger.info("update.video.usecase", message="Founded entity", entity=entity.to_dict())

        entity.update(
            data={
                "title": input_params.title,
                "categories": input_params.categories,
                "genres": input_params.genres,
                "slug": input_params.slug,
                "year_launched": input_params.year_launched,
                "opened": input_params.opened,
                "rating": input_params.rating,
                "duration": input_params.duration,
                "is_deleted": input_params.is_deleted,
            }
        )

        if input_params.status == ACTIVE_STATUS:
            entity.activate()

        if input_params.status == INACTIVE_STATUS:
            entity.deactivate()

        if self.logger:
            self.logger.info("update.video.usecase", message="Entity updated", entity=entity.to_dict())

        self.repo.update(entity=entity)
        if self.logger:
            self.logger.info("create.video.usecase", message="Entity saved")

        return self.__to_output(video=entity)

    def __to_output(self, video: Video):
        return VideoOutputMapper.to_output(klass=UpdateVideoOutput, video=video)



# Python
from typing import List, Generic, TypeVar, Optional
from dataclasses import dataclass

# Apps
from kernel_catalogo_videos.core.domain.repositories import SearchResult

Filter = TypeVar("Filter")
Item = TypeVar("Item")


@dataclass(slots=True, frozen=True)
class SearchInput(Generic[Filter]):
    page: Optional[int] = None
    per_page: Optional[int] = None
    sort: Optional[str] = None
    sort_direction: Optional[str] = None
    filters: Optional[Filter] = None


@dataclass(slots=True, frozen=True)
class PaginationOutput(Generic[Item, Filter]):
    items: List[Item]
    current_page: int
    per_page: int
    last_page: int
    sort: Optional[str]
    sort_direction: Optional[str]
    filters: Optional[Filter]
    total: int


Output = TypeVar("Output", bound=PaginationOutput)


@dataclass(slots=True, frozen=True)
class PaginationOutputMapper:
    output_child: Output

    @staticmethod
    def from_child(output_child: Output):
        return PaginationOutputMapper(output_child)

    def to_output(self, items: List[Item], result: SearchResult) -> PaginationOutput[Item, Filter]:
        return self.output_child(
            items=items,
            current_page=result.current_page,
            per_page=result.per_page,
            last_page=result.last_page,
            sort=result.sort,
            sort_direction=result.sort_direction,
            filters=result.filters,
            total=result.total,
        )



"""
Caso de uso para criar um membro
"""
# Python
from logging import Logger
from dataclasses import asdict

# Apps
from kernel_catalogo_videos.core.application.use_case import UseCase
from kernel_catalogo_videos.castmembers.domain.entities import CastMember
from kernel_catalogo_videos.castmembers.domain.factories import CastMemberFactory
from kernel_catalogo_videos.castmembers.domain.repositories import CastMemberRepository
from kernel_catalogo_videos.castmembers.application.use_cases.dto import CastMemberOutputMapper
from kernel_catalogo_videos.castmembers.application.use_cases.create.input import CreateCastMemberInput
from kernel_catalogo_videos.castmembers.application.use_cases.create.output import CreateCastMemberOutput


class CreateGenreUseCase(UseCase[CreateCastMemberInput, CreateCastMemberOutput]):
    """
    Classe para criar um membro
    """

    repo: CastMemberRepository

    def __init__(self, repo: CastMemberRepository, logger: Logger | None = None) -> None:
        self.repo = repo
        self.logger = logger

    def save(self, entity: CastMember):
        self.repo.insert(entity)
        if self.logger:
            self.logger.info("create.castmember.usecase", message="Entity saved")

    def execute(self, input_params: CreateCastMemberInput) -> CreateCastMemberOutput:
        if self.logger:
            self.logger.info("create.castmember.usecase", message="Initial Payload", input_params=asdict(input_params))

        castmember_entity = CastMemberFactory(logger=self.logger).create(
            name=input_params.name, kind=input_params.kind, status=input_params.status
        )

        self.save(castmember_entity)

        return self.__to_output(castmember=castmember_entity)

    def __to_output(self, castmember: CastMember):
        return CastMemberOutputMapper.to_output(klass=CreateCastMemberOutput, castmember=castmember)



"""
Input para criar um genero
"""
# Python
from typing import List, Optional
from dataclasses import dataclass

# Apps
from kernel_catalogo_videos.genres.domain.entities import Genre


@dataclass(slots=True, frozen=True)
class CreateGenreInput:
    title: str
    categories: List[str]
    status: Optional[int] = Genre.get_field("status").default



"""
Listar membros
"""


# Python
from logging import Logger
from dataclasses import asdict

# Apps
from kernel_catalogo_videos.core.application.dto import PaginationOutputMapper
from kernel_catalogo_videos.core.domain.repositories import SearchParams, SearchResult
from kernel_catalogo_videos.core.application.use_case import UseCase
from kernel_catalogo_videos.castmembers.domain.repositories import CastMemberRepository
from kernel_catalogo_videos.castmembers.application.use_cases.dto import CastMemberOutputDTO, CastMemberOutputMapper
from kernel_catalogo_videos.castmembers.application.use_cases.search.input import SearchCastMemberInput
from kernel_catalogo_videos.castmembers.application.use_cases.search.output import SearchCastMemberOutput


class SearchCastMembersUseCase(UseCase[SearchCastMemberInput, SearchCastMemberOutput]):
    """
    Classe para listar membros
    """

    repo: CastMemberRepository

    def __init__(self, repo: CastMemberRepository, logger: Logger | None = None) -> None:
        self.repo = repo
        self.logger = logger

    def execute(self, input_params: SearchCastMemberInput) -> SearchCastMemberOutput:
        if self.logger:
            self.logger.info("search.castmember.usecase", message="Initial Payload", input_params=asdict(input_params))

        search_params = SearchParams(**asdict(input_params))

        result = self.repo.search(params=search_params)

        if self.logger:
            self.logger.info("search.castmember.usecase", message="Entities listed")

        return self.__to_output(result=result)

    def __to_output(self, result: SearchResult):
        items = list(
            map(
                lambda castmember: CastMemberOutputMapper.to_output(klass=CastMemberOutputDTO, castmember=castmember),
                result.items,
            )
        )
        return PaginationOutputMapper.from_child(SearchCastMemberOutput).to_output(items=items, result=result)



"""
Listar generos
"""


# Python
from logging import Logger
from dataclasses import asdict

# Apps
from kernel_catalogo_videos.core.application.dto import PaginationOutputMapper
from kernel_catalogo_videos.core.domain.repositories import SearchParams, SearchResult
from kernel_catalogo_videos.core.application.use_case import UseCase
from kernel_catalogo_videos.genres.domain.repositories import GenreRepository
from kernel_catalogo_videos.genres.application.use_cases.dto import GenreOutputDTO, GenreOutputMapper
from kernel_catalogo_videos.genres.application.use_cases.search.input import SearchGenreInput
from kernel_catalogo_videos.genres.application.use_cases.search.output import SearchGenreOutput


class SearchGenresUseCase(UseCase[SearchGenreInput, SearchGenreOutput]):
    """
    Classe para listar generos
    """

    repo: GenreRepository

    def __init__(self, repo: GenreRepository, logger: Logger | None = None) -> None:
        self.repo = repo
        self.logger = logger

    def execute(self, input_params: SearchGenreInput) -> SearchGenreOutput:
        if self.logger:
            self.logger.info("search.genre.usecase", message="Initial Payload", input_params=asdict(input_params))

        search_params = SearchParams(**asdict(input_params))

        result = self.repo.search(params=search_params)

        if self.logger:
            self.logger.info("search.genre.usecase", message="Entities listed")

        return self.__to_output(result=result)

    def __to_output(self, result: SearchResult):
        items = list(
            map(
                lambda genre: GenreOutputMapper.to_output(klass=GenreOutputDTO, genre=genre),
                result.items,
            )
        )
        return PaginationOutputMapper.from_child(SearchGenreOutput).to_output(items=items, result=result)



"""
A domain seedwork for validator Interfaces
"""

# Python
import abc
from typing import Any, Dict, List, Generic, TypeVar
from dataclasses import dataclass

ErrorsField = Dict[str, List[str]]
PropsValidated = TypeVar("PropsValidated")


@dataclass(frozen=True)
class ValidatorFieldInterface(abc.ABC, Generic[PropsValidated]):
    """
    Interface for any Validator instance with a validate method
    """

    errors: ErrorsField = None
    validated_data: PropsValidated = None

    @abc.abstractmethod
    def validate(self, validator: Any):
        """
        Abstract method for validate the data
        """
        raise NotImplementedError()



"""
Modulo seedwork para uma Entidade
"""
# pylint: disable=duplicate-code

# Python
from abc import ABC
from dataclasses import Field, field, asdict, dataclass

# Apps
from kernel_catalogo_videos.core.domain.unique_entity_id import UniqueEntityId


@dataclass(kw_only=True, frozen=True)
class Entity(ABC):
    """
    Classe Entidade
    """

    unique_entity_id: UniqueEntityId = field(default_factory=UniqueEntityId)

    # pylint: disable=C0103
    @property
    def id(self):
        """
        Retorna o id do tipo UUID como string
        """
        return str(self.unique_entity_id)

    def to_dict(self):
        """
        Retorna um dicionario com o id como string
        """
        entity_dict = asdict(self)
        entity_dict.pop("unique_entity_id")
        entity_dict["id"] = self.id

        return entity_dict

    @classmethod
    def get_field(cls, entity_field: str) -> Field:
        # pylint: disable=no-member
        return cls.__dataclass_fields__[entity_field]

    def _set(self, field_name, value):
        object.__setattr__(self, field_name, value)

        return self



"""
Define um ou mais validators para categoria
"""

# Python

# Apps
from kernel_catalogo_videos.core.domain.validators import PropsValidated, ValidatorFieldInterface


class DummyValidator(ValidatorFieldInterface[PropsValidated]):
    """
    Validação de serializers do django rest framework
    """

    def validate(self, validator: bool) -> bool:
        if not validator:
            return False

        return True



"""
Caso de uso para atualizar uma categoria
"""
# pylint: disable=duplicate-code

# Python
from logging import Logger
from dataclasses import asdict

# Apps
from kernel_catalogo_videos.core.utils import ACTIVE_STATUS, INACTIVE_STATUS
from kernel_catalogo_videos.core.application.use_case import UseCase
from kernel_catalogo_videos.categories.domain.entities import Category
from kernel_catalogo_videos.categories.domain.repositories import CategoryRepository
from kernel_catalogo_videos.categories.application.use_cases.dto import CategoryOutputMapper
from kernel_catalogo_videos.categories.application.use_cases.update.input import UpdateCategoryInput
from kernel_catalogo_videos.categories.application.use_cases.update.output import UpdateCategoryOutput


class UpdateCategoryUseCase(UseCase[UpdateCategoryInput, UpdateCategoryOutput]):
    """
    Atualizar uma categoria
    """

    repo: CategoryRepository

    def __init__(self, repo: CategoryRepository, logger: Logger | None = None) -> None:
        self.repo = repo
        self.logger = logger

    def execute(self, input_params: UpdateCategoryInput) -> UpdateCategoryOutput:
        if self.logger:
            self.logger.info("update.category.usecase", message="Initial Payload", input_params=asdict(input_params))

        entity = self.repo.find_by_id(input_params.id)

        if self.logger:
            self.logger.info("update.category.usecase", message="Founded entity", entity=entity.to_dict())

        entity.update(data={"title": input_params.title, "description": input_params.description})

        if input_params.status == ACTIVE_STATUS:
            entity.activate()

        if input_params.status == INACTIVE_STATUS:
            entity.deactivate()

        if self.logger:
            self.logger.info("update.category.usecase", message="Entity updated", entity=entity.to_dict())

        self.repo.update(entity=entity)
        if self.logger:
            self.logger.info("create.category.usecase", message="Entity saved")

        return self.__to_output(category=entity)

    def __to_output(self, category: Category):
        return CategoryOutputMapper.to_output(klass=UpdateCategoryOutput, category=category)



"""
Caso de uso para criar uma categoria
"""
# Python
from logging import Logger
from dataclasses import asdict

# Apps
from kernel_catalogo_videos.videos.domain.entities import Video
from kernel_catalogo_videos.core.application.use_case import UseCase
from kernel_catalogo_videos.videos.domain.repositories import VideoRepository
from kernel_catalogo_videos.videos.application.use_cases.dto import VideoOutputMapper
from kernel_catalogo_videos.videos.application.use_cases.create.input import CreateVideoInput
from kernel_catalogo_videos.videos.application.use_cases.create.output import CreateVideoOutput


class CreateVideoUseCase(UseCase[CreateVideoInput, CreateVideoOutput]):
    """
    Classe para criar um video
    """

    repo: VideoRepository

    def __init__(self, repo: VideoRepository, logger: Logger | None = None) -> None:
        self.repo = repo
        self.logger = logger

    def execute(self, input_params: CreateVideoInput) -> CreateVideoOutput:
        if self.logger:
            self.logger.info("create.video.usecase", message="Initial Payload", input_params=asdict(input_params))

        video = Video(
            title=input_params.title,
            categories=input_params.categories,
            genres=input_params.genres,
            slug=input_params.slug,
            year_launched=input_params.year_launched,
            opened=input_params.opened,
            rating=input_params.rating,
            duration=input_params.duration,
            is_deleted=input_params.is_deleted,
            status=input_params.status,
        )
        video.normalize()

        if self.logger:
            self.logger.info("create.video.usecase", message="Entity created", video=video.to_dict())

        self.repo.insert(video)
        if self.logger:
            self.logger.info("create.video.usecase", message="Entity saved")

        return self.__to_output(video=video)

    def __to_output(self, video: Video):
        return VideoOutputMapper.to_output(klass=CreateVideoOutput, video=video)



"""
Input para criar um video
"""
# Python
from typing import List, Optional
from dataclasses import dataclass

# Apps
from kernel_catalogo_videos.videos.domain.entities import Video


@dataclass(slots=True, frozen=True)
class CreateVideoInput:
    title: str
    categories: List[str]
    genres: List[str]
    slug: Optional[str] = Video.get_field("slug").default
    year_launched: int = Video.get_field("year_launched").default
    opened: bool = Video.get_field("opened").default
    rating: str = Video.get_field("rating").default
    duration: int = Video.get_field("duration").default
    status: Optional[int] = Video.get_field("status").default
    is_deleted: Optional[bool] = Video.get_field("is_deleted").default



"""
Caso de uso para atualizar um membro
"""
# pylint: disable=duplicate-code

# Python
from logging import Logger
from dataclasses import asdict

# Apps
from kernel_catalogo_videos.core.utils import ACTIVE_STATUS, INACTIVE_STATUS
from kernel_catalogo_videos.core.application.use_case import UseCase
from kernel_catalogo_videos.castmembers.domain.entities import CastMember
from kernel_catalogo_videos.castmembers.domain.repositories import CastMemberRepository
from kernel_catalogo_videos.castmembers.application.use_cases.dto import CastMemberOutputMapper
from kernel_catalogo_videos.castmembers.application.use_cases.update.input import UpdateCastMemberInput
from kernel_catalogo_videos.castmembers.application.use_cases.update.output import UpdateCastMemberOutput


class UpdateCastMemberUseCase(UseCase[UpdateCastMemberInput, UpdateCastMemberOutput]):
    """
    Atualizar um membro
    """

    repo: CastMemberRepository

    def __init__(self, repo: CastMemberRepository, logger: Logger | None = None) -> None:
        self.repo = repo
        self.logger = logger

    def execute(self, input_params: UpdateCastMemberInput) -> UpdateCastMemberOutput:
        if self.logger:
            self.logger.info("update.castmember.usecase", message="Initial Payload", input_params=asdict(input_params))

        entity = self.repo.find_by_id(input_params.id)

        if self.logger:
            self.logger.info("update.castmember.usecase", message="Founded entity", entity=entity.to_dict())

        entity.update(data={"title": input_params.title})

        if input_params.status == ACTIVE_STATUS:
            entity.activate()

        if input_params.status == INACTIVE_STATUS:
            entity.deactivate()

        if self.logger:
            self.logger.info("update.castmember.usecase", message="Entity updated", entity=entity.to_dict())

        self.repo.update(entity=entity)
        if self.logger:
            self.logger.info("create.castmember.usecase", message="Entity saved")

        return self.__to_output(castmember=entity)

    def __to_output(self, castmember: CastMember):
        return CastMemberOutputMapper.to_output(klass=UpdateCastMemberOutput, castmember=castmember)



"""
Caso de uso para atualizar um genero
"""
# pylint: disable=duplicate-code

# Python
from logging import Logger
from dataclasses import asdict

# Apps
from kernel_catalogo_videos.core.utils import ACTIVE_STATUS, INACTIVE_STATUS
from kernel_catalogo_videos.genres.domain.entities import Genre
from kernel_catalogo_videos.core.application.use_case import UseCase
from kernel_catalogo_videos.genres.domain.repositories import GenreRepository
from kernel_catalogo_videos.genres.application.use_cases.dto import GenreOutputMapper
from kernel_catalogo_videos.genres.application.use_cases.update.input import UpdateGenreInput
from kernel_catalogo_videos.genres.application.use_cases.update.output import UpdateGenreOutput


class UpdateGenreUseCase(UseCase[UpdateGenreInput, UpdateGenreOutput]):
    """
    Atualizar um genero
    """

    repo: GenreRepository

    def __init__(self, repo: GenreRepository, logger: Logger | None = None) -> None:
        self.repo = repo
        self.logger = logger

    def execute(self, input_params: UpdateGenreInput) -> UpdateGenreOutput:
        if self.logger:
            self.logger.info("update.genre.usecase", message="Initial Payload", input_params=asdict(input_params))

        entity = self.repo.find_by_id(input_params.id)

        if self.logger:
            self.logger.info("update.genre.usecase", message="Founded entity", entity=entity.to_dict())

        entity.update(data={"title": input_params.title})

        if input_params.status == ACTIVE_STATUS:
            entity.activate()

        if input_params.status == INACTIVE_STATUS:
            entity.deactivate()

        if self.logger:
            self.logger.info("update.genre.usecase", message="Entity updated", entity=entity.to_dict())

        self.repo.update(entity=entity)
        if self.logger:
            self.logger.info("create.genre.usecase", message="Entity saved")

        return self.__to_output(genre=entity)

    def __to_output(self, genre: Genre):
        return GenreOutputMapper.to_output(klass=UpdateGenreOutput, genre=genre)



"""
Modulo seedwork representando um id do tipo UUID
"""
# Python
import uuid
from dataclasses import field, dataclass

# Apps
from kernel_catalogo_videos.core.utils import uuidv4
from kernel_catalogo_videos.core.domain.exceptions import InvalidUUIDException


@dataclass(frozen=True)
class UniqueEntityId:
    """
    Classe entidade representando um id como UUID
    """

    # pylint: disable=C0103
    id: str = field(default_factory=uuidv4)

    def __post_init__(self):
        id_value = str(self.id) if isinstance(self.id, uuid.UUID) else self.id
        object.__setattr__(self, "id", id_value)
        self.__validate()

    def __validate(self):
        try:
            uuid.UUID(self.id)
        except ValueError as exc:
            raise InvalidUUIDException() from exc

    def __str__(self):
        return f"{self.id}"



# Python
import logging

# Third
import structlog


def configure_logging(debug=False):

    default = [
        structlog.contextvars.merge_contextvars,
        structlog.processors.add_log_level,
        structlog.processors.TimeStamper(),
        structlog.processors.StackInfoRenderer(),
        structlog.processors.CallsiteParameterAdder(
            {
                structlog.processors.CallsiteParameter.FILENAME,
                structlog.processors.CallsiteParameter.FUNC_NAME,
                structlog.processors.CallsiteParameter.LINENO,
                structlog.processors.CallsiteParameter.PATHNAME,
            }
        ),
    ]
    production = default + [structlog.processors.dict_tracebacks, structlog.processors.JSONRenderer()]

    development = default + [
        structlog.dev.set_exc_info,
        structlog.dev.ConsoleRenderer(exception_formatter=structlog.dev.rich_traceback),
    ]

    processors = development if debug else production

    structlog.configure(
        processors=processors,
        wrapper_class=structlog.make_filtering_bound_logger(logging.NOTSET),
        context_class=dict,
        logger_factory=structlog.PrintLoggerFactory(),
        cache_logger_on_first_use=False,
    )


def make_logger(debug: bool):
    configure_logging(debug)
    log = structlog.get_logger()
    return log



"""
Define factories para o dominio Category
"""


class CategoryValidatorFactory:
    """
    Representa uma factory pattern para criar um validator
    """

    @staticmethod
    def create(serializer_class):
        """
        Instancia uma classe do tipo Validator
        """
        return serializer_class()



"""
Modulo seedwork para repositories classes
"""

# Python
import abc
import math
from typing import Any, List, Generic, TypeVar, Optional
from dataclasses import Field, field, dataclass

# Apps
from kernel_catalogo_videos.core.domain.entities import Entity
from kernel_catalogo_videos.core.domain.exceptions import NotFoundException
from kernel_catalogo_videos.core.domain.unique_entity_id import UniqueEntityId

ET = TypeVar("ET", bound=Entity)
Input = TypeVar("Input")
Output = TypeVar("Output")
Filter = TypeVar("Filter", str, Any)


class RepositoryInterface(Generic[ET], abc.ABC):
    """
    Interface para definir o contrato dos metodos
    """

    @abc.abstractmethod
    def insert(self, entity: ET) -> None:
        """
        Inserir uma entidade
        """
        raise NotImplementedError()

    @abc.abstractmethod
    def find_by_id(self, entity_id: str | UniqueEntityId) -> ET:
        """
        Procurar uma entidade por id
        """
        raise NotImplementedError()

    @abc.abstractmethod
    def find_all(self) -> List[ET]:
        """
        Buscar todas as entidades
        """
        raise NotImplementedError()

    @abc.abstractmethod
    def update(self, entity: ET) -> None:
        """
        Atualizar uma entidade
        """
        raise NotImplementedError()

    @abc.abstractmethod
    def delete(self, entity_id: str | UniqueEntityId) -> None:
        """
        Deletar uma entidade por id
        """
        raise NotImplementedError()


class SearchableRepositoryInterface(Generic[ET, Input, Output], RepositoryInterface[ET], abc.ABC):
    """
    Contrato para implementar uma busca
    """

    sortable_fields: List[str] = []

    @abc.abstractmethod
    def search(self, params: Input) -> Output:
        """
        Buscar um item de acordo com os parametros
        """
        raise NotImplementedError()


@dataclass(slots=True, kw_only=True)
class SearchParams(Generic[Filter]):
    """
    Definição de parametros de busca genérico
    """

    page: Optional[int] = 1
    per_page: Optional[int] = 10
    sort: Optional[str] = None
    sort_direction: Optional[str] = None
    filters: Optional[Filter] = None

    def __post_init__(self):
        self._normalize_page()
        self._normalize_per_page()
        self._normalize_sort()
        self._normalize_sort_direction()
        self._normalize_filters()

    def _normalize_page(self):
        page = self._convert_to_int(self.page)
        if page <= 0:
            page = self._get_dataclass_field("page").default

        self.page = page

    def _normalize_per_page(self):
        per_page = self._convert_to_int(self.per_page, default=10)

        if per_page <= 0:
            per_page = self._get_dataclass_field("per_page").default

        self.per_page = per_page

    def _normalize_sort(self):
        self.sort = None if self.sort == "" or self.sort is None else str(self.sort)

    def _normalize_sort_direction(self):
        if not self.sort:
            self.sort_direction = None

        sort_direction = str(self.sort_direction).lower()

        self.sort_direction = "asc" if sort_direction not in ["asc", "desc"] else sort_direction

    def _normalize_filters(self):
        self.filters = None if self.filters == "" or self.filters is None else str(self.filters)

    def _convert_to_int(self, value: Any, default=1) -> int:
        try:
            return int(value)
        except (ValueError, TypeError):
            return default

    def _get_dataclass_field(self, field_name):
        # pylint: disable=no-member
        return SearchParams.__dataclass_fields__[field_name]

    @classmethod
    def get_field(cls, entity_field: str) -> Field:
        # pylint: disable=no-member
        return cls.__dataclass_fields__[entity_field]


@dataclass(slots=True, kw_only=True, frozen=True)
class SearchResult(Generic[ET, Filter]):
    """
    Definição resultados de uma busca genérico
    """

    items: List[ET]
    current_page: int
    per_page: int
    last_page: int = field(init=False)
    sort: Optional[str] = None
    sort_direction: Optional[str] = None
    filters: Optional[Filter] = None
    total: int = 0

    def __post_init__(self):
        object.__setattr__(self, "last_page", math.ceil(self.total / self.per_page))

    def to_dict(self):
        """
        Retorna um dicionario com os dados instanciados
        """
        return {
            "items": self.items,
            "total": self.total,
            "current_page": self.current_page,
            "per_page": self.per_page,
            "last_page": self.last_page,
            "sort": self.sort,
            "sort_direction": self.sort_direction,
            "filters": self.filters,
        }


@dataclass(slots=True)
class InMemoryRepository(RepositoryInterface[ET], abc.ABC):
    """
    Repositorio com as principais operações CRUD em memória
    """

    items: List[ET] = field(default_factory=lambda: [])

    def insert(self, entity: ET) -> None:
        self.items.append(entity)

    def find_by_id(self, entity_id: str | UniqueEntityId) -> ET:
        entity_id_str = str(entity_id)

        return self._get(entity_id_str)

    def find_all(self) -> List[ET]:
        return self.items

    def update(self, entity: ET) -> None:

        entity_founded = self._get(entity.id)
        index = self.items.index(entity_founded)
        self.items[index] = entity

    def delete(self, entity_id: str | UniqueEntityId) -> None:
        entity_id_str = str(entity_id)
        entity_founded = self._get(entity_id_str)
        self.items.remove(entity_founded)

    def _get(self, entity_id: str):
        entity = next(filter(lambda item: item.id == entity_id, self.items), None)

        if entity is None:
            raise NotFoundException(f"Entity not found using ID '{entity_id}'")

        return entity


class InMemorySearchableRepository(
    Generic[ET, Filter],
    InMemoryRepository[ET],
    SearchableRepositoryInterface[ET, SearchParams[Filter], SearchResult[ET, Filter]],
    abc.ABC,
):
    """
    Repositório que herda as principais operacoes de CRUD mais o SEARCH
    """

    def search(self, params: SearchParams[Filter]) -> SearchResult[ET, Filter]:
        items_filtered = self._apply_filter(self.items, params.filters)
        items_sorted = self._apply_sort(items_filtered, params.sort, params.sort_direction)
        items_paginated = self._apply_paginate(items_sorted, params.page, params.per_page)

        return SearchResult(
            items=items_paginated,
            total=len(items_filtered),
            current_page=params.page,
            per_page=params.per_page,
            sort=params.sort,
            sort_direction=params.sort_direction,
            filters=params.filters,
        )

    @abc.abstractmethod
    def _apply_filter(self, items: List[ET], filters: Optional[Filter]):
        raise NotImplementedError()

    def _apply_sort(self, items: List[ET], sort: Optional[str], sort_direction: Optional[str]) -> List[ET]:
        if sort and sort in self.sortable_fields:
            is_reverse = sort_direction == "desc"
            return sorted(items, key=lambda item: getattr(item, sort), reverse=is_reverse)

        return items

    def _apply_paginate(self, items: List[ET], page: int, per_page: int) -> List[ET]:
        start = (page - 1) * per_page
        limit = start + per_page
        return items[slice(start, limit)]



"""
Modulo para definir tudo que represente uma ligação com o rabbitmq
"""
# Python
from abc import ABC, abstractmethod
from json import dumps
from typing import Dict

# Third
import pika
from pika import URLParameters, BlockingConnection


class RabbitMQ:
    """
    A class to do a connection
    """

    @staticmethod
    def connect(amqp_uri: str):
        """
        Connect sync to RabbitMQ and returns the connection and channel
        """
        params = URLParameters(amqp_uri)
        # number of socket connection attempts
        params.connection_attempts = 7
        # interval between socket connection attempts; see also connection_attempts.
        params.retry_delay = 300
        # AMQP connection heartbeat timeout value for negotiation during connection
        # tuning or callable which is invoked during connection tuning
        params.heartbeat = 600
        # None or float blocked connection timeout
        params.blocked_connection_timeout = 300
        try:
            connect = BlockingConnection(params)
            channel = connect.channel()

            return connect, channel

        except Exception as exc:
            raise exc


class InterfacePublishToQueue(ABC):
    """
    Interface to publish messages
    """

    conn = None
    channel = None
    exchange = None
    exchange_dlx = None
    queue_dl = None
    routing_key_dl = None
    queue = None
    routing_key = None

    @abstractmethod
    def __init__(self):
        pass

    @abstractmethod
    def connect(self):
        """
        Abstract method to connect
        """
        pass

    @abstractmethod
    def publish(self, message: dict):
        """
        Abstract method to publish
        """
        pass

    def close_channel(self):
        """
        Close a channel
        """
        self.channel.close()

    def close_connection(self):
        """
        Close connection
        """
        self.conn.close()

    def teardown(self):
        """
        Closes a channel and connection
        """
        self.close_channel()
        self.close_connection()

    def run(self, message):
        """
        Connect and publish a message
        """
        self.connect()
        self.publish(message)

    def message_to_json(self, message: Dict):
        """ "
        Transforms a message dict to a json
        """
        try:
            body = dumps(message)
            return body
        except Exception as exc:
            raise exc


class InterfaceRabbitMQ(InterfacePublishToQueue):
    """
    Interface with connect and publish methods
    """

    def connect(self):
        if self.exchange_dlx and self.queue_dl and self.routing_key_dl:
            arguments = {
                "x-dead-letter-exchange": self.exchange_dlx,
                "x-dead-letter-routing-key": self.routing_key_dl,
            }
        else:
            arguments = {}

        self.channel.queue_declare(queue=self.queue, durable=True, arguments=arguments)
        self.channel.queue_bind(self.queue, self.exchange, routing_key=self.routing_key)
        # Turn on delivery confirmations
        self.channel.confirm_delivery()

    def publish(self, message: dict):
        """
        Publish a message in default exchange
        """

        properties = pika.BasicProperties(
            app_id="admin-catalog-video",
            content_type="application/json",
            delivery_mode=pika.DeliveryMode.Persistent,
        )

        body = self.message_to_json(message)

        # Send a message
        try:
            self.channel.basic_publish(
                exchange=self.exchange,
                routing_key=self.routing_key,
                body=body,
                properties=properties,
            )

        except pika.exceptions.UnroutableError as err:
            raise err



# Python
import uuid
from typing import Any
from datetime import datetime, timezone

utc = timezone.utc

INACTIVE_STATUS = 0
ACTIVE_STATUS = 1

KIND_DIRECTOR = 0
KIND_ACTOR = 1


def now() -> datetime:
    return datetime.now(tz=utc)


def uuidv4() -> uuid.UUID:
    return uuid.uuid4()


def check_is_deleted(obj: Any):
    return obj.is_deleted is True


def check_is_inactive(obj: Any) -> bool:
    return obj.status == INACTIVE_STATUS


def check_is_inactive_or_deleted(obj: Any) -> bool:
    is_deleted = check_is_deleted(obj) if hasattr(obj, "is_deleted") else False
    is_inactive = check_is_inactive(obj) if hasattr(obj, "status") else False

    return is_inactive or is_deleted



"""
Caso de uso para criar uma categoria
"""
# Python
from logging import Logger
from dataclasses import asdict

# Apps
from kernel_catalogo_videos.core.application.use_case import UseCase
from kernel_catalogo_videos.categories.domain.entities import Category
from kernel_catalogo_videos.categories.domain.repositories import CategoryRepository
from kernel_catalogo_videos.categories.application.use_cases.dto import CategoryOutputMapper
from kernel_catalogo_videos.categories.application.use_cases.create.input import CreateCategoryInput
from kernel_catalogo_videos.categories.application.use_cases.create.output import CreateCategoryOutput


class CreateCategoryUseCase(UseCase[CreateCategoryInput, CreateCategoryOutput]):
    """
    Classe para criar uma categoria
    """

    repo: CategoryRepository

    def __init__(self, repo: CategoryRepository, logger: Logger | None = None) -> None:
        self.repo = repo
        self.logger = logger

    def execute(self, input_params: CreateCategoryInput) -> CreateCategoryOutput:
        if self.logger:
            self.logger.info("create.category.usecase", message="Initial Payload", input_params=asdict(input_params))

        category = Category(
            title=input_params.title,
            description=input_params.description,
            status=input_params.status,
        )
        category.normalize()
        if self.logger:
            self.logger.info("create.category.usecase", message="Entity created", category=category.to_dict())

        self.repo.insert(category)
        if self.logger:
            self.logger.info("create.category.usecase", message="Entity saved")

        return self.__to_output(category=category)

    def __to_output(self, category: Category):
        return CategoryOutputMapper.to_output(klass=CreateCategoryOutput, category=category)



"""
Retorna uma lista de videos
"""

# Apps
from kernel_catalogo_videos.core.application.dto import PaginationOutput
from kernel_catalogo_videos.videos.application.use_cases.dto import VideoOutputDTO


class SearchVideoOutput(PaginationOutput[VideoOutputDTO, str]):
    pass



"""
Retorna uma lista de membros
"""

# Apps
from kernel_catalogo_videos.core.application.dto import PaginationOutput
from kernel_catalogo_videos.castmembers.application.use_cases.dto import CastMemberOutputDTO


class SearchCastMemberOutput(PaginationOutput[CastMemberOutputDTO, str]):
    pass



"""
Output retornando os dados de uma categoria
"""
# pylint: disable=duplicate-code

# Python
from typing import List, TypeVar, Optional
from datetime import datetime
from dataclasses import dataclass

# Apps
from kernel_catalogo_videos.genres.domain.entities import Genre


@dataclass(slots=True, frozen=True)
class GenreOutputDTO:
    id: str  # pylint: disable=invalid-name
    categories: List[str]
    title: str
    slug: str
    is_deleted: bool
    created_at: datetime
    status: Optional[int] = 1


Output = TypeVar("Output", bound=GenreOutputDTO)


class GenreOutputMapper:
    @staticmethod
    def to_output(klass: Output, genre: Genre) -> Output:
        return klass(
            id=genre.id,
            categories=genre.categories,
            title=genre.title,
            slug=genre.slug,
            status=genre.status,
            is_deleted=genre.is_deleted,
            created_at=genre.created_at,
        )



"""
Input para criar um genero
"""
# Python
from typing import Optional
from dataclasses import dataclass

# Apps
from kernel_catalogo_videos.castmembers.domain.entities import CastMember


@dataclass(slots=True, frozen=True)
class CreateCastMemberInput:
    name: str
    kind: int = CastMember.get_field("kind").default
    status: Optional[int] = CastMember.get_field("status").default



"""
Input para retornar um genero
"""

# Python
from dataclasses import dataclass

# Apps
from kernel_catalogo_videos.videos.application.use_cases.dto import VideoOutputDTO


@dataclass(slots=True, frozen=True)
class UpdateVideoOutput(VideoOutputDTO):
    pass



# Apps
from kernel_catalogo_videos.categories.application.use_cases.dto import CategoryOutputDTO, CategoryOutputMapper
from kernel_catalogo_videos.categories.application.use_cases.get.input import GetCategoryInput
from kernel_catalogo_videos.categories.application.use_cases.get.output import GetCategoryOutput
from kernel_catalogo_videos.categories.application.use_cases.create.input import CreateCategoryInput
from kernel_catalogo_videos.categories.application.use_cases.delete.input import DeleteCategoryInput
from kernel_catalogo_videos.categories.application.use_cases.get.use_case import GetCategoryUseCase
from kernel_catalogo_videos.categories.application.use_cases.search.input import SearchCategoryInput
from kernel_catalogo_videos.categories.application.use_cases.update.input import UpdateCategoryInput
from kernel_catalogo_videos.categories.application.use_cases.create.output import CreateCategoryOutput
from kernel_catalogo_videos.categories.application.use_cases.search.output import SearchCategoryOutput
from kernel_catalogo_videos.categories.application.use_cases.update.output import UpdateCategoryOutput
from kernel_catalogo_videos.categories.application.use_cases.create.use_case import CreateCategoryUseCase
from kernel_catalogo_videos.categories.application.use_cases.delete.use_case import DeleteCategoryUseCase
from kernel_catalogo_videos.categories.application.use_cases.search.use_case import SearchCategoriesUseCase
from kernel_catalogo_videos.categories.application.use_cases.update.use_case import UpdateCategoryUseCase



"""
Input para atualizar um genero
"""
# Python
from typing import List, Optional
from dataclasses import dataclass

# Apps
from kernel_catalogo_videos.genres.domain.entities import Genre


@dataclass(slots=True, frozen=False)
class UpdateGenreInput:
    id: str  # pylint: disable=invalid-name
    categories: List[str]
    title: str
    slug: Optional[str] = Genre.get_field("slug").default
    status: Optional[int] = Genre.get_field("status").default
    is_deleted: Optional[bool] = Genre.get_field("is_deleted").default



"""
Retorna uma lista de generos
"""

# Apps
from kernel_catalogo_videos.core.application.dto import PaginationOutput
from kernel_catalogo_videos.genres.application.use_cases.dto import GenreOutputDTO


class SearchGenreOutput(PaginationOutput[GenreOutputDTO, str]):
    pass



"""
Define a entidade de Categoria
"""
# pylint: disable=duplicate-code


# Python
from typing import Any, Dict, Optional
from datetime import datetime
from dataclasses import field, dataclass

# Third
from slugify import slugify

# Apps
from kernel_catalogo_videos.core.utils import ACTIVE_STATUS, INACTIVE_STATUS, now
from kernel_catalogo_videos.core.domain.entities import Entity


@dataclass(kw_only=True, frozen=True, slots=True)
class Category(Entity):
    """
    Representa a entidade categoria e seus dados
    """

    title: str
    slug: Optional[str] = ""
    description: Optional[str] = ""
    status: Optional[int] = ACTIVE_STATUS
    is_deleted: bool = False
    created_at: Optional[datetime] = field(default_factory=now)

    def __post_init__(self):
        self.normalize()

    def update(self, data: Dict[str, Any]):
        """
        Atualiza os dados internos da entidade
        """
        for field_name, value in data.items():
            self._set(field_name, value)

        self.normalize()

    def activate(self):
        """
        Seta o atributo status como ativo
        """
        self._set("status", ACTIVE_STATUS)

    def deactivate(self):
        """
        Seta o atributo status como inativo
        """
        self._set("status", INACTIVE_STATUS)

    def normalize(self):
        slugged = slugify(self.title)
        self._set("slug", slugged)



"""
Caso de uso para criar uma categoria
"""
# Python
from logging import Logger
from dataclasses import asdict

# Apps
from kernel_catalogo_videos.genres.domain.entities import Genre
from kernel_catalogo_videos.genres.domain.factories import GenreFactory
from kernel_catalogo_videos.core.application.use_case import UseCase
from kernel_catalogo_videos.categories.domain.services import CategoryService
from kernel_catalogo_videos.genres.domain.repositories import GenreRepository
from kernel_catalogo_videos.genres.application.use_cases.dto import GenreOutputMapper
from kernel_catalogo_videos.genres.application.use_cases.create.input import CreateGenreInput
from kernel_catalogo_videos.genres.application.use_cases.create.output import CreateGenreOutput


class CreateGenreUseCase(UseCase[CreateGenreInput, CreateGenreOutput]):
    """
    Classe para criar um genero
    """

    repo: GenreRepository

    def __init__(self, repo: GenreRepository, category_service: CategoryService, logger: Logger | None = None) -> None:
        self.repo = repo
        self.category_service = category_service
        self.logger = logger

    def save(self, entity):
        self.repo.insert(entity)
        if self.logger:
            self.logger.info("create.genre.usecase", message="Entity saved")

    def check_categories(self, input_params: CreateGenreInput):
        self.category_service.check_categories_exists(categories_uuid=input_params.categories)
        self.category_service.check_categories_is_inactive(categories_uuid=input_params.categories)
        self.category_service.check_categories_is_deleted(categories_uuid=input_params.categories)

    def execute(self, input_params: CreateGenreInput) -> CreateGenreOutput:
        if self.logger:
            self.logger.info("create.genre.usecase", message="Initial Payload", input_params=asdict(input_params))

        self.check_categories(input_params=input_params)

        genre_entity = GenreFactory(logger=self.logger).create(
            categories=input_params.categories, title=input_params.title, status=input_params.status
        )

        self.save(genre_entity)

        return self.__to_output(genre=genre_entity)

    def __to_output(self, genre: Genre):
        return GenreOutputMapper.to_output(klass=CreateGenreOutput, genre=genre)



# Python
from typing import List
from logging import Logger

# Apps
from kernel_catalogo_videos.categories.domain.repositories import CategoryRepository


class CategoryService:
    repo: CategoryRepository

    def __init__(self, repo: CategoryRepository, logger: Logger | None = None) -> None:
        self.repo = repo
        self.logger = logger

    def check_categories_exists(self, categories_uuid: List[str]):
        # TODO: Check categories exists
        if self.logger:
            self.logger.info(
                "services.category.check_categories_exists",
                message="Check categories exists",
                categories=categories_uuid,
            )

    def check_categories_is_inactive(self, categories_uuid: List[str]):
        # TODO: Check categories is inactive
        if self.logger:
            self.logger.info(
                "services.category.check_categories_is_inactive",
                message="Check categories exists",
                categories=categories_uuid,
            )

    def check_categories_is_deleted(self, categories_uuid: List[str]):
        # TODO: Check categories is deleted
        if self.logger:
            self.logger.info(
                "services.category.check_categories_is_deleted",
                message="Check categories is deleted",
                categories=categories_uuid,
            )



"""
Buscar uma  categoria
"""
# pylint: disable=duplicate-code
# Python
from logging import Logger
from dataclasses import asdict

# Apps
from kernel_catalogo_videos.core.application.use_case import UseCase
from kernel_catalogo_videos.categories.domain.entities import Category
from kernel_catalogo_videos.categories.domain.repositories import CategoryRepository
from kernel_catalogo_videos.categories.application.use_cases.dto import CategoryOutputMapper
from kernel_catalogo_videos.categories.application.use_cases.get.input import GetCategoryInput
from kernel_catalogo_videos.categories.application.use_cases.get.output import GetCategoryOutput


class GetCategoryUseCase(UseCase[GetCategoryInput, GetCategoryOutput]):
    """
    Classe para criar uma categoria
    """

    repo: CategoryRepository

    def __init__(self, repo: CategoryRepository, logger: Logger | None = None) -> None:
        self.repo = repo
        self.logger = logger

    def execute(self, input_params: GetCategoryInput) -> GetCategoryOutput:
        if self.logger:
            self.logger.info("get.category.usecase", message="Initial Payload", input_params=asdict(input_params))

        category = self.repo.find_by_id(input_params.id)

        if self.logger:
            self.logger.info("get.category.usecase", message="Entity founded")

        return self.__to_output(category=category)

    def __to_output(self, category: Category):
        return CategoryOutputMapper.to_output(klass=GetCategoryOutput, category=category)



"""
Input para retornar um genero
"""

# Python
from dataclasses import dataclass

# Apps
from kernel_catalogo_videos.genres.application.use_cases.dto import GenreOutputDTO


@dataclass(slots=True, frozen=True)
class CreateGenreOutput(GenreOutputDTO):
    pass



"""
Caso de uso para criar um membro
"""
# Python
from logging import Logger
from dataclasses import asdict

# Apps
from kernel_catalogo_videos.core.application.use_case import UseCase
from kernel_catalogo_videos.castmembers.domain.repositories import CastMemberRepository
from kernel_catalogo_videos.castmembers.application.use_cases.delete.input import DeleteCastMemberInput


class DeleteCastMemberUseCase(UseCase[DeleteCastMemberInput, None]):
    """
    Classe para deletar um membro
    """

    repo: CastMemberRepository

    def __init__(self, repo: CastMemberRepository, logger: Logger | None = None) -> None:
        self.repo = repo
        self.logger = logger

    def execute(self, input_params: DeleteCastMemberInput) -> None:
        if self.logger:
            self.logger.info("delete.castmember.usecase", message="Initial Payload", input_params=asdict(input_params))

        # pylint: disable=unexpected-keyword-arg
        self.repo.delete(input_params.id)

        if self.logger:
            self.logger.info("delete.castmember.usecase", message="Entity deleted")



# Python
from enum import Enum


class Messages(Enum):
    FIELD_REQUIRED = "Campo obrigatório."
    INVALID_DATA = "Ocorreu um erro nos campos informados."
    DOES_NOT_EXIST = "Este(a) {} não existe."
    EXCEPTION = "Ocorreu um erro no servidor. Contate o administrador."
    ALREADY_EXISTS = "Já existe um(a) {} com estes dados."
    NO_DATA = "Nenhum dado foi postado."
    PASSWORD_DIDNT_MATCH = "As senhas não conferem."  # nosec
    PASSWORD_CHANGED_OK = "A senha foi alterada com sucesso."  # nosec
    USER_INACTIVE = "Este usuário está inativo."
    USER_HAS_NO_PERMISSIONS = "Este usuário não possui permissões necessárias."

    RESOURCE_CREATED = "{} criado(a)."
    RESOURCE_FETCHED_PAGINATED = "Lista os/as {} paginados(as)."
    RESOURCE_FETCHED = "{} retornado(a)."
    RESOURCE_UPDATED = "{} atualizado(a)."
    RESOURCE_DELETED = "{} deletado(a)."

    TOKEN_CREATED = "Token criado."  # nosec
    INVALID_CREDENTIALS = "As credenciais estão inválidas para log in."  # nosec
    TOKEN_EXPIRED = "Token expirou."  # nosec
    PERMISSION_DENIED = "Permissão negada."

    RESOURCE_NOT_ALLOWED = "O método solicitado não é permitido para este endpoint."
    RESOURCE_NOT_FOUND = "Recurso não encontrado."
    RESOURCE_BAD_REQUEST = "Houve um erro na requisição."
    RESOURCE_DOES_NOT_EXIST = "Um recurso com este ID não existe mais."
    RESOURCE_USER_ALREADY_EXISTS = "Um usuário com este username já existe."

    PARAM_IS_STRING = "O parâmetro {} precisa ser um string"
    PARAM_IS_DICT = "O parâmetro {} precisa ser um dict"
    PARAM_IS_INT = "O parâmetro {} precisa ser um int"



"""
Buscar um video
"""
# pylint: disable=duplicate-code
# Python
from logging import Logger
from dataclasses import asdict

# Apps
from kernel_catalogo_videos.videos.domain.entities import Video
from kernel_catalogo_videos.core.application.use_case import UseCase
from kernel_catalogo_videos.videos.domain.repositories import VideoRepository
from kernel_catalogo_videos.videos.application.use_cases.dto import VideoOutputMapper
from kernel_catalogo_videos.videos.application.use_cases.get.input import GetVideoInput
from kernel_catalogo_videos.videos.application.use_cases.get.output import GetVideoOutput


class GetVideoUseCase(UseCase[GetVideoInput, GetVideoOutput]):
    """
    Classe para criar um video
    """

    repo: VideoRepository

    def __init__(self, repo: VideoRepository, logger: Logger | None = None) -> None:
        self.repo = repo
        self.logger = logger

    def execute(self, input_params: GetVideoInput) -> GetVideoOutput:
        if self.logger:
            self.logger.info("get.video.usecase", message="Initial Payload", input_params=asdict(input_params))

        video = self.repo.find_by_id(input_params.id)

        if self.logger:
            self.logger.info("get.video.usecase", message="Entity founded")

        return self.__to_output(video=video)

    def __to_output(self, video: Video):
        return VideoOutputMapper.to_output(klass=GetVideoOutput, video=video)



"""
Input para criar uma categoria
"""
# Python
from typing import Optional
from dataclasses import dataclass

# Apps
from kernel_catalogo_videos.categories.domain.entities import Category


@dataclass(slots=True, frozen=True)
class CreateCategoryInput:
    title: str
    description: Optional[str] = Category.get_field("description").default
    status: Optional[int] = Category.get_field("status").default



# Python
from logging import Logger

# Apps
from kernel_catalogo_videos.castmembers.domain.entities import CastMember


class CastMemberFactory:
    def __init__(self, logger: Logger | None = None) -> None:
        self.logger = logger

    def create(self, name: str, kind: str, status: int) -> CastMember:

        castmember = CastMember(name=name, kind=kind, status=status)

        if self.logger:
            self.logger.info("create.castmember.factory", message="Entity created", castmember=castmember.to_dict())

        return castmember



"""
Output retornando os dados de uma categoria
"""
# pylint: disable=duplicate-code

# Python
from typing import TypeVar, Optional
from datetime import datetime
from dataclasses import dataclass

# Apps
from kernel_catalogo_videos.castmembers.domain.entities import CastMember


@dataclass(slots=True, frozen=True)
class CastMemberOutputDTO:
    id: str  # pylint: disable=invalid-name
    name: str
    kind: int
    is_deleted: bool
    created_at: datetime
    status: Optional[int] = 1


Output = TypeVar("Output", bound=CastMemberOutputDTO)


class CastMemberOutputMapper:
    @staticmethod
    def to_output(klass: Output, castmember: CastMember) -> Output:
        return klass(
            id=castmember.id,
            name=castmember.name,
            kind=castmember.kind,
            status=castmember.status,
            is_deleted=castmember.is_deleted,
            created_at=castmember.created_at,
        )



"""
Input para atualizar um genero
"""
# Python
from typing import Optional
from dataclasses import dataclass

# Apps
from kernel_catalogo_videos.castmembers.domain.entities import CastMember


@dataclass(slots=True, frozen=False)
class UpdateCastMemberInput:
    id: str  # pylint: disable=invalid-name
    name: str
    kind: int = CastMember.get_field("kind").default
    status: Optional[int] = CastMember.get_field("status").default
    is_deleted: Optional[bool] = CastMember.get_field("is_deleted").default



"""
Retorna uma lista de categorias
"""

# Apps
from kernel_catalogo_videos.core.application.dto import PaginationOutput
from kernel_catalogo_videos.categories.application.use_cases.dto import CategoryOutputDTO


class SearchCategoryOutput(PaginationOutput[CategoryOutputDTO, str]):
    pass



# Apps
from kernel_catalogo_videos.core.infrastructure.rabbitmq import InterfaceRabbitMQ


class PublishCreatedVideoToQueue(InterfaceRabbitMQ):
    """
    Classe que inicializa as configurações do rabbitmq e a fila
    """

    def __init__(
        self,
        rabbitmq_conn,
        rabbitmq_channel,
        exchange,
        exchange_dlx,
        catalog_videos_dead,
        catalog_videos_dead_rk,
        catalog_videos,
        catalog_videos_rk,
    ):
        self.conn = rabbitmq_conn
        self.channel = rabbitmq_channel
        self.exchange = exchange
        self.exchange_dlx = exchange_dlx
        self.queue_dl = catalog_videos_dead
        self.routing_key_dl = catalog_videos_dead_rk
        self.queue = catalog_videos
        self.routing_key = catalog_videos_rk
        self.connect()
        super().__init__()



# Python
from typing import List
from logging import Logger

# Apps
from kernel_catalogo_videos.genres.domain.entities import Genre


class GenreFactory:
    def __init__(self, logger: Logger | None = None) -> None:
        self.logger = logger

    def create(self, categories: List[str], title: str, status: int) -> Genre:

        genre = Genre(
            categories=categories,
            title=title,
            status=status,
        )
        genre.normalize()

        if self.logger:
            self.logger.info("create.genre.usecase", message="Entity created", genre=genre.to_dict())

        return genre



"""
Input para retornar uma categoria
"""

# Python
from dataclasses import dataclass

# Apps
from kernel_catalogo_videos.categories.application.use_cases.dto import CategoryOutputDTO


@dataclass(slots=True, frozen=True)
class CreateCategoryOutput(CategoryOutputDTO):
    pass



"""
Output retornando os dados de uma categoria
"""
# pylint: disable=duplicate-code

# Python
from typing import List, TypeVar, Optional
from datetime import datetime
from dataclasses import dataclass

# Apps
from kernel_catalogo_videos.videos.domain.entities import RATING_EIGHTEEN_YEARS, Video


@dataclass(slots=True, frozen=True)
class VideoOutputDTO:
    id: str  # pylint: disable=invalid-name
    title: str
    slug: str
    categories: List[str]
    genres: List[str]
    is_deleted: bool
    created_at: datetime
    year_launched: int = 2022
    opened: bool = True
    rating: str = RATING_EIGHTEEN_YEARS
    duration: int = 50
    status: Optional[int] = 1
    thumb_file: Optional[str] = ""
    banner_file: Optional[str] = ""
    trailer_file: Optional[str] = ""
    video_file: Optional[str] = ""


Output = TypeVar("Output", bound=VideoOutputDTO)


class VideoOutputMapper:
    @staticmethod
    def to_output(klass: Output, video: Video) -> Output:
        return klass(
            id=video.id,
            title=video.title,
            slug=video.slug,
            year_launched=video.slug,
            opened=video.opened,
            rating=video.rating,
            duration=video.duration,
            thumb_file=video.thumb_file,
            banner_file=video.banner_file,
            trailer_file=video.trailer_file,
            video_file=video.video_file,
            status=video.status,
            is_deleted=video.is_deleted,
            created_at=video.created_at,
        )



"""
Output retornando os dados de uma categoria
"""

# Python
from typing import TypeVar, Optional
from datetime import datetime
from dataclasses import dataclass

# Apps
from kernel_catalogo_videos.categories.domain.entities import Category


@dataclass(slots=True, frozen=True)
class CategoryOutputDTO:
    id: str  # pylint: disable=invalid-name
    title: str
    slug: str
    is_deleted: bool
    created_at: datetime
    description: Optional[str] = None
    status: Optional[int] = 1


Output = TypeVar("Output", bound=CategoryOutputDTO)


class CategoryOutputMapper:
    @staticmethod
    def to_output(klass: Output, category: Category) -> Output:
        return klass(
            id=category.id,
            title=category.title,
            slug=category.slug,
            description=category.description,
            status=category.status,
            is_deleted=category.is_deleted,
            created_at=category.created_at,
        )



"""
Define os repositories para membros
"""
# Python
from typing import List, Optional

# Apps
from kernel_catalogo_videos.core.domain.repositories import Filter, InMemorySearchableRepository
from kernel_catalogo_videos.castmembers.domain.entities import CastMember
from kernel_catalogo_videos.castmembers.domain.repositories import CastMemberRepository


class CastMemberInMemoryRepository(CastMemberRepository, InMemorySearchableRepository):
    """
    Classe para tratar um repositorio em memória

    TODO: Refatorar para InMemorySearchableRepository
    """

    sortable_fields: List[str] = ["name", "created_at"]

    def _apply_filter(self, items: List[CastMember], filters: Optional[Filter]):
        if filters:
            items_filtered = filter(lambda item: filters.lower() in item.name.lower(), items)
            return list(items_filtered)

        return items

    def _apply_sort(
        self, items: List[CastMember], sort: Optional[str], sort_direction: Optional[str]
    ) -> List[CastMember]:
        return (
            super()._apply_sort(items, sort=sort, sort_direction=sort_direction)
            if sort
            else super()._apply_sort(items, sort="created_at", sort_direction="desc")
        )



"""
Input para atualizar uma categoria
"""
# Python
from typing import Optional
from dataclasses import dataclass

# Apps
from kernel_catalogo_videos.categories.domain.entities import Category


@dataclass(slots=True, frozen=False)
class UpdateCategoryInput:
    id: str  # pylint: disable=invalid-name
    title: str
    slug: Optional[str] = Category.get_field("slug").default
    description: Optional[str] = Category.get_field("description").default
    status: Optional[int] = Category.get_field("status").default
    is_deleted: Optional[bool] = Category.get_field("is_deleted").default



class OperationDBError(Exception):
    def __init__(self, exc, operation: str, code: int = 500, entity=None, **kwargs) -> None:
        self.code = code
        self.exc = exc
        self.entity = entity
        self.operation = operation
        self.extra = kwargs
        self.message = str(exc)
        super().__init__(f"{operation} Error")



"""
Buscar um genero
"""
# pylint: disable=duplicate-code
# Python
from logging import Logger
from dataclasses import asdict

# Apps
from kernel_catalogo_videos.core.application.use_case import UseCase
from kernel_catalogo_videos.castmembers.domain.entities import CastMember
from kernel_catalogo_videos.castmembers.domain.repositories import CastMemberRepository
from kernel_catalogo_videos.castmembers.application.use_cases.dto import CastMemberOutputMapper
from kernel_catalogo_videos.castmembers.application.use_cases.get.input import GetCastMemberInput
from kernel_catalogo_videos.castmembers.application.use_cases.get.output import GetCastMemberOutput


class GetCastMemberUseCase(UseCase[GetCastMemberInput, GetCastMemberOutput]):
    """
    Classe para criar um membro
    """

    repo: CastMemberRepository

    def __init__(self, repo: CastMemberRepository, logger: Logger | None = None) -> None:
        self.repo = repo
        self.logger = logger

    def execute(self, input_params: GetCastMemberInput) -> GetCastMemberOutput:
        if self.logger:
            self.logger.info("get.castmember.usecase", message="Initial Payload", input_params=asdict(input_params))

        castmember = self.repo.find_by_id(input_params.id)

        if self.logger:
            self.logger.info("get.castmember.usecase", message="Entity founded")

        return self.__to_output(castmember=castmember)

    def __to_output(self, castmember: CastMember):
        return CastMemberOutputMapper.to_output(klass=GetCastMemberOutput, castmember=castmember)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['kernel_catalogo_videos',
 'kernel_catalogo_videos.castmembers',
 'kernel_catalogo_videos.castmembers.application',
 'kernel_catalogo_videos.castmembers.application.use_cases',
 'kernel_catalogo_videos.castmembers.application.use_cases.create',
 'kernel_catalogo_videos.castmembers.application.use_cases.delete',
 'kernel_catalogo_videos.castmembers.application.use_cases.get',
 'kernel_catalogo_videos.castmembers.application.use_cases.search',
 'kernel_catalogo_videos.castmembers.application.use_cases.update',
 'kernel_catalogo_videos.castmembers.domain',
 'kernel_catalogo_videos.castmembers.infrastructure',
 'kernel_catalogo_videos.categories',
 'kernel_catalogo_videos.categories.application',
 'kernel_catalogo_videos.categories.application.use_cases',
 'kernel_catalogo_videos.categories.application.use_cases.create',
 'kernel_catalogo_videos.categories.application.use_cases.delete',
 'kernel_catalogo_videos.categories.application.use_cases.get',
 'kernel_catalogo_videos.categories.application.use_cases.search',
 'kernel_catalogo_videos.categories.application.use_cases.update',
 'kernel_catalogo_videos.categories.domain',
 'kernel_catalogo_videos.categories.infrastructure',
 'kernel_catalogo_videos.core',
 'kernel_catalogo_videos.core.application',
 'kernel_catalogo_videos.core.domain',
 'kernel_catalogo_videos.core.infrastructure',
 'kernel_catalogo_videos.genres',
 'kernel_catalogo_videos.genres.application',
 'kernel_catalogo_videos.genres.application.use_cases',
 'kernel_catalogo_videos.genres.application.use_cases.create',
 'kernel_catalogo_videos.genres.application.use_cases.delete',
 'kernel_catalogo_videos.genres.application.use_cases.get',
 'kernel_catalogo_videos.genres.application.use_cases.search',
 'kernel_catalogo_videos.genres.application.use_cases.update',
 'kernel_catalogo_videos.genres.domain',
 'kernel_catalogo_videos.genres.infrastructure',
 'kernel_catalogo_videos.videos',
 'kernel_catalogo_videos.videos.application',
 'kernel_catalogo_videos.videos.application.use_cases',
 'kernel_catalogo_videos.videos.application.use_cases.create',
 'kernel_catalogo_videos.videos.application.use_cases.delete',
 'kernel_catalogo_videos.videos.application.use_cases.get',
 'kernel_catalogo_videos.videos.application.use_cases.search',
 'kernel_catalogo_videos.videos.application.use_cases.update',
 'kernel_catalogo_videos.videos.domain',
 'kernel_catalogo_videos.videos.infrastructure']

package_data = \
{'': ['*']}

install_requires = \
['pika>=1.3.1,<2.0.0',
 'python-slugify>=7.0.0,<8.0.0',
 'structlog>=22.3.0,<23.0.0']

setup_kwargs = {
    'name': 'kernel-catalogo-videos',
    'version': '0.1.9',
    'description': '',
    'long_description': None,
    'author': 'Lucas',
    'author_email': 'lucassrod@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



"""
Como exemplo de Value Objects podemos citar objetos que representam
Nome, Endereco, Email, Dinheiro, etc.

Um objeto Dinheiro com valor R$ 5,00 é igual a outro objeto de mesmo valor;
um objeto Nome com valor 'Jose' é igual a outro objeto de mesmo valor não
importando a instância do objeto.

Os Value Objects devem ser imutáveis e pequenos e
representam algo único como quantidades, descrições simples, valores,

Ser imutável
Seu único uso é como propriedade de uma ou mais entidades ou mesmo outro VO;
Saber usar todas as suas propriedades ao realizar qualquer tipo de
verificação de igualdade, incluindo aquelas que
baseiam sua comparação em códigos hash;

Sua lógica não deve ter efeitos colaterais no estado fora do VO;

"""

# Python
import json
from abc import ABC
from dataclasses import fields, dataclass


@dataclass(frozen=True)
class ValueObject(ABC):
    """
    Representa o  str de uma entidade
    """

    def __str__(self) -> str:
        fields_names = [field.name for field in fields(self)]
        return (
            str(getattr(self, fields_names[0]))
            if len(fields_names) == 1
            else json.dumps({field_name: getattr(self, field_name) for field_name in fields_names})
        )



"""
Principais Exceções lançadas pela aplicação
"""
# Apps
from kernel_catalogo_videos.core.domain.validators import ErrorsField


class InvalidUUIDException(Exception):
    """
    Representa um UUID invalido
    """

    def __init__(self, error: str = "Id must be a valid UUID") -> None:
        super().__init__(error)


class ValidationException(Exception):
    """
    Representa erro de validação generico
    """

    pass


class EntityValidationException(Exception):
    """
    Representa uma entidade invalida
    """

    def __init__(self, error: ErrorsField) -> None:
        self.error = error
        super().__init__("Entity Validation Error")


class NotFoundException(Exception):
    """
    Representa uma entidade não encontrada
    """

    pass



"""
Input para retornar um genero
"""

# Python
from dataclasses import dataclass

# Apps
from kernel_catalogo_videos.castmembers.application.use_cases.dto import CastMemberOutputDTO


@dataclass(slots=True, frozen=True)
class UpdateCastMemberOutput(CastMemberOutputDTO):
    pass



"""
Define os repositories para a categoria
"""
# Python
import abc

# Apps
from kernel_catalogo_videos.core.domain.repositories import SearchParams as DefaultSearchParams
from kernel_catalogo_videos.core.domain.repositories import SearchResult as DefaultSearchResult
from kernel_catalogo_videos.core.domain.repositories import SearchableRepositoryInterface

# Local
from .entities import CastMember


class _SearchParams(DefaultSearchParams):
    """
    CastMember Search Result
    Privadas do modulo
    Sendo utilizadas com o typing da classe e da innerclass
    """

    pass


class _SearchResult(DefaultSearchResult):
    """
    CastMember Search Result
    Privadas do modulo
    Sendo utilizadas com o typing da classe e da innerclass
    """

    pass


class CastMemberRepository(SearchableRepositoryInterface[CastMember, _SearchParams, _SearchResult], abc.ABC):
    """
    Classe para tratar um membro
    """

    SearchParams = _SearchParams
    SearchResult = _SearchResult
